package com.nova.flutter_ads;

import android.app.Activity;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.ark.adkit.basics.models.ArkVideoCallbacks;
import com.ark.adkit.polymers.polymer.ADTool;


import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BasicMessageChannel;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.StringCodec;

/**
 * FlutterAdsPlugin
 */
public class FlutterAdsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private BasicMessageChannel rewardRunTimeSender;
    private BasicMessageChannel insertRunTimeSender;
    private FlutterPluginBinding pluginBinding;
    private Activity activity;


    @Override
    public void onMethodCall(MethodCall call, Result result) {

        Log.d("wangbang", "" + call.method);
        switch (call.method) {
            case "initKvConfig":
                ADTool.initKvConfig();
                result.success(true);
                break;
            case "initRewardVideo":
                initRewardAd(call, result);
                break;
            case "initInsertVideo":
                initInsertAd(call, result);
                break;
            case "initRewardVideoDirect":
                initRewardAdDirect(call, result);
                break;
            case "initInsertVideoDirect":
                initInsertAdDirect(call, result);
                break;
            case "preInitVideo":
                ADTool.getADTool().getManager().getVideoWrapper().preInitReward(activity);
                break;
            case "setConfig":
                ADTool.Builder builder = new ADTool.Builder();
                Object bannerWidth = call.argument("bannerWidth");
                Object nativeWidth = call.argument("nativeWidth");
                if (bannerWidth != null) {
                    builder.setTtadBannerWidth((Integer) bannerWidth);
                }
                if (nativeWidth != null) {
                    builder.setTtadNativeWidth((Integer) nativeWidth);
                }
                ADTool.initialize(builder.build());
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_ads");
        channel.setMethodCallHandler(this);
        pluginBinding = flutterPluginBinding;
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {

    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        activity = binding.getActivity();
        setupViews();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivity() {
    }

    private void setupViews() {

        //注册开屏广告UI插件
        SplashViewFactory factory = new SplashViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/splashview", factory);

        //注册本地广告
        NativeViewFactory nativeViewFactory = new NativeViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/nativeview", nativeViewFactory);

        //注册banner广告
        BannerViewFactory bannerViewFactory = new BannerViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/bannerview", bannerViewFactory);

    }

//    private static void setupViews(Registrar registrar) {
//        //注册开屏广告UI插件
//        SplashViewFactory factory = new SplashViewFactory(registrar.activity(), registrar.messenger());
//        registrar.platformViewRegistry().registerViewFactory("plugins.nova.ads/splashview", factory);
//
//        //注册本地广告
//        NativeViewFactory nativeViewFactory = new NativeViewFactory(registrar.activity(), registrar.messenger());
//        registrar.platformViewRegistry().registerViewFactory("plugins.nova.ads/nativeview", nativeViewFactory);
//
//        //注册banner广告
//        BannerViewFactory bannerViewFactory = new BannerViewFactory(registrar.activity(), registrar.messenger());
//        registrar.platformViewRegistry().registerViewFactory("plugins.nova.ads/bannerview", bannerViewFactory);
//
//    }


    Handler handler;

    private void initInsertAd(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        BinaryMessenger binaryMessenger = pluginBinding.getBinaryMessenger();

        insertRunTimeSender = new BasicMessageChannel<>(binaryMessenger, "plugins.nova.ads.event/insert", StringCodec.INSTANCE);
        ADTool.getADTool().getManager().getVideoWrapper().loadFull(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose(boolean isRewarded) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }


    private void initInsertAdDirect(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        BinaryMessenger binaryMessenger = pluginBinding.getBinaryMessenger();

        insertRunTimeSender = new BasicMessageChannel<>(binaryMessenger, "plugins.nova.ads.event/insert", StringCodec.INSTANCE);
        ADTool.getADTool().getManager().getVideoWrapper().loadFullDirect(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose(boolean isRewarded) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }


            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }


    private void initRewardAd(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        BinaryMessenger binaryMessenger = pluginBinding.getBinaryMessenger();

        rewardRunTimeSender = new BasicMessageChannel<>(binaryMessenger, "plugins.nova.ads.event/reward", StringCodec.INSTANCE);

        ADTool.getADTool().getManager().getVideoWrapper().loadReward(activity, new ArkVideoCallbacks() {

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose(boolean isRewarded) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }
        });
    }


    private void initRewardAdDirect(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        BinaryMessenger binaryMessenger = pluginBinding.getBinaryMessenger();
        rewardRunTimeSender = new BasicMessageChannel<>(binaryMessenger, "plugins.nova.ads.event/reward", StringCodec.INSTANCE);

        ADTool.getADTool().getManager().getVideoWrapper().loadRewardDirect(activity, new ArkVideoCallbacks() {

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose(boolean isRewarded) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }
        });
    }
}
